from sqlalchemy import *

from dataProviderBase import *
from T0Mon.utils import *
from T0Mon.consts import *
from T0Mon.sqlManager import *

from T0Mon.globals import logger
import T0Mon.config

### Module for classes providing data about T0 files ###

class BaseRepackDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id", decorator = buildRepackLink),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
#        Col("Status"),            # TODO : check these columns in the database, add to the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = "Repacked files"

    def _buildCountQuery(self):
	wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)

	self.countQuery = """ SELECT COUNT(DISTINCT f.id)
				FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
					INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
					INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)  
                                         """ + self.fromCond + """
				WHERE dt.name = 'RAW' """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)
	primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

	self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
				FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
					INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
					 """ + self.fromCond + """ 
                                WHERE dt.name = 'RAW' """ + self.whereCond + """ 
				ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order

class RepackByIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

        repackedTable = Tbl(TABLES.REPACKED)

        self.whereCond = repackedTable.c.repacked_id.in_(ids)

        BaseRepackDP._load(self)

class RepackByRunIdDP(BaseRepackDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)
        self.title = "Temporarily Repacked files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

	self.whereCond = """ and m.run = """ + str(runid) + """ """
	
	self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) """

        BaseRepackDP._load(self)

class RepackByRepackMergeIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

        repackedTable = Tbl(TABLES.REPACKED)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)

        self.whereCond = \
            or_(
                repackedTable.c.repacked_id.in_(select([parentageTable.c.input_id], parentageTable.c.output_id.in_(ids))),
                and_(
                    repackedTable.c.repacked_id.in_(ids),
                    not_(repackedTable.c.repacked_id.in_(select([parentageTable.c.output_id])))
                )
            )
        BaseRepackDP._load(self)

class RepackByRepackJobIdsDP(BaseRepackDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackDP.__init__(self, orderby, asc, page)

	repackStreamerAssocTable = Tbl(TABLES.REPACK_STREAMER_ASSOC, True)
	jobDatasetStreamerAssocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC, True)
	wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)

	ids_str = ""
	for id in ids:
		ids_str += str(id)

        self.whereCond = """ AND f.id IN (SELECT repacked_id 
						FROM """ + repackStreamerAssocTable + """ 
						WHERE streamer_id IN (SELECT streamer_id 
									FROM """ + jobDatasetStreamerAssocTable + """ 
									WHERE job_id IN (""" + ids_str + """))
						) 
			     AND dp.primary_dataset IN (SELECT primary_dataset_id
							FROM """ + jobDatasetStreamerAssocTable + """
                                                        WHERE job_id IN (""" + ids_str + """))"""
	
	self.fromCond = """ """

        BaseRepackDP._load(self)

class BaseRepackMergeDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id", decorator = buildRepackMergeLink),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
#        Col("Status"),    # TODO : check this columns from database for the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = "Repacked files"

    def _buildCountQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsWorkflowTable = Tbl(TABLES.WMBS_WORKFLOW, True)

        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
					INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid) 
                                         """ + self.fromCond + """
                                WHERE dt.name = 'RAW' and c.subscription = (SELECT id
                                                                              FROM """ + wmbsSubscriptionTable + """
                                                                              WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
                                                                              AND workflow = (SELECT id FROM """ + wmbsWorkflowTable + """ WHERE spec = 'FileMerge')
                                                                           ) """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        dataTierTable = Tbl(TABLES.DATA_TIER, True)
	primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
        wmbsWorkflowTable = Tbl(TABLES.WMBS_WORKFLOW, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT  DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + dataTierTable + """ dt ON (dp.data_tier = dt.id)
					INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id) 
                                        INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                         """ + self.fromCond + """
                                WHERE dt.name = 'RAW' and c.subscription = (SELECT id
                                                                              FROM """ + wmbsSubscriptionTable + """
                                                                              WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
                                                                              AND workflow = (SELECT id FROM """ + wmbsWorkflowTable + """ WHERE spec = 'FileMerge')
                                                                           ) """ + self.whereCond + """ 
				ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order

class RepackMergeByRecoJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.RECO_JOB_REPACK_ASSOC)
        self.whereCond = repackedTable.c.repacked_id.in_(
            select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
        )
        BaseRepackMergeDP._load(self)

class RepackMergeByIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        self.whereCond = repackedTable.c.repacked_id.in_(ids)
        BaseRepackMergeDP._load(self)

class RepackMergeByRepackMergeJobIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)
        self.whereCond = repackedTable.c.repacked_id.in_(
            select([parentageTable.c.output_id], parentageTable.c.input_id.in_(
                select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
            ))
        )
        BaseRepackMergeDP._load(self)

class RepackMergeByRepackIdsDP(BaseRepackMergeDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)
        repackedTable = Tbl(TABLES.REPACKED)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_MERGE_PARENTAGE)
        self.whereCond = \
            and_(
                or_(
                    repackedTable.c.status == REPACK_RECONSTRUCTION_QUEUED,
                    repackedTable.c.status == REPACK_RECONSTRUCTION_DISABLED
                ),
                or_(
                    repackedTable.c.repacked_id.in_(
                        select([parentageTable.c.output_id],
                            parentageTable.c.input_id.in_(ids)
                        )
                    ),
                    repackedTable.c.repacked_id.in_(ids)
                )
            )
        if len(ids) != 0:
            BaseRepackMergeDP._load(self)

class RepackMergeByRunIdDP(BaseRepackMergeDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRepackMergeDP.__init__(self, orderby, asc, page)

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) """

        BaseRepackMergeDP._load(self)


class BaseRecoDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id", decorator=buildRecoLink),
        Col("Filesize", decorator=humanFilesize),
        Col("Events"),
#        Col("Status"),    # TODO : check these columns in the database for the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = "Reco files"

    def _buildCountQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                FROM """ + wmbsFileDetailsTable + """ f 
					INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                         """ + self.fromCond + """
                                WHERE c.subscription = (SELECT id
                                                           FROM """ + wmbsSubscriptionTable + """
                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Reconstructable')
                                                           ) """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
	primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
					INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
					INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                         """ + self.fromCond + """
                                WHERE c.subscription = (SELECT id
                                                           FROM """ + wmbsSubscriptionTable + """
                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Reconstructable')
                                                           ) """ + self.whereCond + """ 
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RecoMergeByRunIdDP(BaseRecoDP):
        cols = Cols([
                Col("Id"),
                Col("Filesize", decorator = humanFilesize),
                Col("Events"),
#                Col("Status"),    # TODO : check these columns in the database for the final version of this table
#                Col("Export status"),
                Col("Dataset"),
                Col("Lfn", decorator = lambda x: textToImageAlt(x)),
                ])

        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseRecoDP.__init__(self, orderby, asc, page)
		self.title = "Reco files"
		self.runid = runid

                BaseRecoDP._load(self)

	def _buildCountQuery(self):
	        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
	        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
	        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
	
	        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
				        FROM """ + wmbsFileDetailsTable + """ f
                                                INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
	                                        INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
	                                WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
					                                                           FROM """ + wmbsSubscriptionTable + """
	                        				                                   WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
												   )
                                        AND f.id IN (SELECT f.id
                                                     FROM """ + wmbsFileDetailsTable + """ f
                                                      INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                      INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                                     WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
                                                                                           FROM """ + wmbsSubscriptionTable + """
                                                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Reconstructable')
                                                                                           )
                                                     )"""

	def _buildMainQuery(self):
	        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
	        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
	        datasetTable = Tbl(TABLES.DATASET_PATH, True)
	        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
	        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
	        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
	        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        	if self.asc:
	                order = "ASC"
	        else:
	                order = "DESC"
	
	        self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                        FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                                INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                                INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
                                                INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
					WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
				                                                                   FROM """ + wmbsSubscriptionTable + """
                                				                                   WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
                                                                				   )
						AND f.id IN (SELECT f.id
				                                FROM """ + wmbsFileDetailsTable + """ f 
								INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
								INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
				                                WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
	                        			                                   FROM """ + wmbsSubscriptionTable + """
	                                                			           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Reconstructable')
				                                                           )
								)
					 ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order

class BaseAlcaskimDP(PagedTableDataProvider):
    cols = Cols([
        Col("Id", decorator=buildRecoLink),
        Col("Filesize", decorator=humanFilesize),
        Col("Events"),
#        Col("Status"),    # TODO : check these columns in the database for the final version of this table
#        Col("Export status"),
        Col("Dataset"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = "Alcaskim files"

    def _buildCountQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                FROM """ + wmbsFileDetailsTable + """ f
                                        INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                         """ + self.fromCond + """
                                WHERE c.subscription = (SELECT id
                                                           FROM """ + wmbsSubscriptionTable + """
                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'AlcaSkimmable')
                                                           ) """ + self.whereCond + """ """


    def _buildMainQuery(self):
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
        wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
        wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                        INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                        INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
                                        INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                         """ + self.fromCond + """
                                WHERE c.subscription = (SELECT id
                                                           FROM """ + wmbsSubscriptionTable + """
                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'AlcaSkimmable')
                                                           ) """ + self.whereCond + """
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class AlcaskimByRunIdDP(BaseAlcaskimDP):
        cols = Cols([
                Col("Id"),
                Col("Filesize", decorator = humanFilesize),
                Col("Events"),
#                Col("Status"),    # TODO : check these columns in the database for the final version of this table
#                Col("Export status"),
                Col("Dataset"),
                Col("Lfn", decorator = lambda x: textToImageAlt(x)),
                ])

        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
	        self.title = "Temporarily Alcaskim files"

		wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

	        self.whereCond = """ AND m.run = """ + str(runid) + """ """

        	self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) """

                BaseAlcaskimDP._load(self)


class AlcaskimMergeByRunIdDP(BaseAlcaskimDP):
        cols = Cols([
                Col("Id"),
                Col("Filesize", decorator = humanFilesize),
                Col("Events"),
#                Col("Status"),    # TODO : check these columns in the database for the final version of this table
#                Col("Export status"),
                Col("Dataset"),
                Col("Lfn", decorator = lambda x: textToImageAlt(x)),
                ])

        def __init__(self, runid, orderby = 0, asc = True, page = 0):
                BaseAlcaskimDP.__init__(self, orderby, asc, page)
                self.title = "Alcaskim files"
		self.runid = runid

                BaseAlcaskimDP._load(self)

        def _buildCountQuery(self):
                wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
                wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
                wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)
                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

                self.countQuery = """ SELECT COUNT(DISTINCT f.id)
                                        FROM """ + wmbsFileDetailsTable + """ f
                                                INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                        WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
                                                                                                   FROM """ + wmbsSubscriptionTable + """
                                                                                                   WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
                                                                                                   )
                                        AND f.id IN (SELECT f.id
                                                     FROM """ + wmbsFileDetailsTable + """ f
                                                      INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                      INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                                     WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
                                                                                           FROM """ + wmbsSubscriptionTable + """
                                                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'AlcaSkimmable')
                                                                                           )
                                                     )"""

        def _buildMainQuery(self):
                wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
                wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
                datasetTable = Tbl(TABLES.DATASET_PATH, True)
                primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsSubFilesCompleteTable = Tbl(TABLES.WMBS_SUB_FILES_COMPLETE, True)
                wmbsSubscriptionTable = Tbl(TABLES.WMBS_SUBSCRIPTION, True)
                wmbsFilesetTable = Tbl(TABLES.WMBS_FILESET, True)

                if self.asc:
                        order = "ASC"
                else:
                        order = "DESC"

                self.mainQuery = """ SELECT DISTINCT f.id, f.filesize, f.events, pd.name, f.lfn
                                        FROM """ + wmbsFileDetailsTable + """ f INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = f.id)
                                                INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id)
                                                INNER JOIN """ + primaryDatasetTable + """ pd ON (dp.primary_dataset = pd.id)
                                                INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                        WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
                                                                                                   FROM """ + wmbsSubscriptionTable + """
                                                                                                   WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'Mergeable')
                                                                                                   )
                                                AND f.id IN (SELECT f.id
                                                                FROM """ + wmbsFileDetailsTable + """ f
                                                                INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                                                INNER JOIN """ + wmbsSubFilesCompleteTable + """ c ON (f.id = c.fileid)
                                                                WHERE m.run = """ + str(self.runid) + """ and c.subscription = (SELECT id
                                                                                           FROM """ + wmbsSubscriptionTable + """
                                                                                           WHERE fileset = (SELECT id FROM """ + wmbsFilesetTable + """ WHERE name = 'AlcaSkimmable')
                                                                                           )
                                                                )
                                         ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order



class RecoByIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        self.whereCond = recoTable.c.reconstructed_id.in_(ids) 
        BaseRecoDP._load(self)

class RecoByRunIdDP(BaseRecoDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        self.title = "Temporarily Reco files"

        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        self.whereCond = """ and m.run = """ + str(runid) + """ """

        self.fromCond = """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid) """

        BaseRecoDP._load(self)

class RecoByRecoJobIdsDP(BaseRecoDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        BaseRecoDP.__init__(self, orderby, asc, page)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        assocTable = Tbl(TABLES.RECO_JOB_REPACK_ASSOC)
        parentageTable = Tbl(TABLES.REPACKED_RECO_PARENTAGE)
        self.whereCond = recoTable.c.reconstructed_id.in_(
            select([parentageTable.c.output_id], parentageTable.c.input_id.in_(
                select([assocTable.c.repacked_id], assocTable.c.job_id.in_(ids))
            ))
        )
        BaseRecoDP._load(self)

class StreamerDP(PagedTableDataProvider):

    cols = Cols([
        Col("Id", decorator = buildStreamerLink),
        Col("Lumi Id"),
        Col("Filesize", decorator = humanFilesize),
        Col("Events"),
        Col("Name"),
        Col("E"),   # TODO : check if this column is still necessary, it doesn't appear in streamer table
#        Col("S"),
        Col("Lfn", decorator = lambda x: textToImageAlt(x))
    ])

    def __init__(self, orderby = 0, asc = True, page = 0):
        PagedTableDataProvider.__init__(self, orderby, asc, page)
        self.title = 'Streamer files'

    def _buildCountQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)

        self.countQuery = select(
            [func.count(streamerTable.c.streamer_id)],
            self.whereCond
        )

    def _buildMainQuery(self):
        streamerTable = Tbl(TABLES.STREAMER)
        streamTable = Tbl(TABLES.STREAM)

        c = (
            streamerTable.c.streamer_id,
            streamerTable.c.lumi_id,
            streamerTable.c.filesize,
            streamerTable.c.events,
            streamTable.c.name,
            streamerTable.c.exportable,
#            streamerTable.c.splitable,
            streamerTable.c.lfn
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
	    from_obj = [streamerTable.outerjoin(streamTable, streamerTable.c.stream_id == streamTable.c.id)]
        ).order_by(o)

class StreamerByRunIdDP(StreamerDP):
    def __init__(self, runid, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.run_id == runid)
        StreamerDP._load(self)

class StreamerByIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        self.whereCond = (streamerTable.c.streamer_id.in_(ids))
        StreamerDP._load(self)

class StreamerByRepackJobIdsDP(StreamerDP):
    def __init__(self, ids, orderby = 0, asc = True, page = 0):
        StreamerDP.__init__(self, orderby, asc, page)
        streamerTable = Tbl(TABLES.STREAMER)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        self.whereCond = streamerTable.c.streamer_id.in_(
            select([func.distinct(assocTable.c.streamer_id)], assocTable.c.job_id.in_(ids))
        )
        StreamerDP._load(self)



